/* Copyright (c) 2010 - 2015 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/
   
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include "va108xx.h"
#include "cyrs15b102.h"
#include "board.h"

/*****************************************************************************/ 
/* Local pre-processor symbols/macros ('#define')                            */ 
/*****************************************************************************/

#define SPI_CMD_RDSR_WIP_Pos	0
#define SPI_CMD_RDSR_WIP_Msk	(0x01UL << SPI_CMD_RDSR_WIP_Pos)

/* Commands */ 
#define SPI_CMD_WREN		0x06UL
#define SPI_CMD_WRDI		0x04UL
#define SPI_CMD_RDSR		0x05UL
#define SPI_CMD_WRSR		0x01UL
#define SPI_CMD_READ		0x03UL
#define SPI_CMD_WRITE	  0x02UL
#define SPI_CMD_RDID		0x9FUL
#define TTZ2564_SE			0x20UL
#define TTZ2564_BE			0x52UL
#define TTZ2564_CE			0x60UL
		    
		
/* Address Masks */ 
#define ADDR_MSB_MASK   (uint32_t)0xFF0000
#define ADDR_MID_MASK   (uint32_t)0x00FF00
#define ADDR_LSB_MASK   (uint32_t)0x0000FF
#define MSB_ADDR_BYTE(addr)   ((uint8_t)((addr & ADDR_MSB_MASK)>>16))
#define MID_ADDR_BYTE(addr)   ((uint8_t)((addr & ADDR_MID_MASK)>>8))
#define LSB_ADDR_BYTE(addr)   ((uint8_t)(addr & ADDR_LSB_MASK))

/* TODO (talexander 16 May 2019):
     -Add check ensureing all while loops terminate, even in case of inresponsive FRAM
     -Add all functions return status code (hal_status_t)
     -Cleanup, formatting improvement
*/

#define INVALID_SPI_BANK  (0xff)

/*****************************************************************************/ 
/* Local variable definitions ('static')                                     */ 
/*****************************************************************************/

static uint8_t SpiBank = INVALID_SPI_BANK; // SPI bank number to use

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

/* 
	Mandatory Flash Programming Functions (Called by FlashOS):
		int Init();					// Initialize Flash 
    int UnInit();  			// De-initialize Flash
    int EraseSector();  // Erase Sector Function
    int ProgramPage();  // Program Page Function
  Optional  Flash Programming Functions (Called by FlashOS):
    int BlankCheck(); 	// Blank Check
    int EraseChip();    // Erase complete Device
    unsigned long Verify(); // Verify Function
  - BlankCheck is necessary if Flash space is not mapped into CPU memory space
  - Verify is necessary if Flash space is not mapped into CPU memory space
  - if EraseChip is not provided than EraseSector for all sectors is called
*/
void wait_idle(void)
{
	while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_TFE_Msk) ) { };	// Wait until TxBuf sends all		
  while( VOR_SPI->BANK[SpiBank].STATUS & SPI_PERIPHERAL_STATUS_BUSY_Msk ) { };	// Wait here until bytes are fully transmitted.
  VOR_SPI->BANK[SpiBank].FIFO_CLR = SPIC_FIFO_CLR_RXFIFO_Msk|SPIC_FIFO_CLR_TXFIFO_Msk;	// Clear Tx & RX fifo 
}

void defeat_WDT(){
#if 0
  // WDT currently not implemented/supported
	WD_FEED_BANK.TOGOUT=1UL<<WD_FEED_PIN;
	//Force-feed WDT with Timer so it won't reset during debug
	//Timers always continue to run and toggle output during debug
	VOR_IOCONFIG->PORTA[WD_FEED_PIN] = (VOR_IOCONFIG->PORTA[WD_FEED_PIN] & ~((0x3)<<IOCONFIG_PORTA_FUNSEL_Pos))	| ((WD_FEED_DEFEAT_FUNSEL)<<IOCONFIG_PORTA_FUNSEL_Pos);
//	VOR_GPIOA_PinMux(WD_FEED_PIN, WD_FEED_DEFEAT_FUNSEL);
	VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1UL << WD_FEED_DEFEAT_TIM_NUM);
  WD_FEED_DEFEAT_TIM->ENABLE = 0x0;
  WD_FEED_DEFEAT_TIM->CTRL = (STAT_SEL_PWMA << TIM_PERIPHERAL_CTRL_STATUS_SEL_Pos);
  WD_FEED_DEFEAT_TIM->RST_VALUE = SYSCLK /10; //0.1sec timer  
	WD_FEED_DEFEAT_TIM->PWMA_VALUE= WD_FEED_DEFEAT_TIM->RST_VALUE/2;
  WD_FEED_DEFEAT_TIM->CNT_VALUE = 0 ;   
  WD_FEED_DEFEAT_TIM->ENABLE = 0x1; // enable seconds timer	
#endif
}

void wait_WIP(void) 
{
	#ifdef CYP15B102_USE_WIP
	int32_t volatile voidRead, readVal, delay;	
  uint32_t sysMsStart;
  
  if(SpiBank == INVALID_SPI_BANK){
    return;
  }
	
	wait_idle();

  sysMsStart = SysTimeMs;
	while(1) { 
		VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_RDSR;	// Read Status Register message
		VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0x00 | SPIC_DATA_BMSTART_BMSTOP_Msk;	// Pump the SPI	
 	  while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_TFE_Msk) ) { };	// Wait until TxBuf sends all		
		while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) { };
	
		voidRead = VOR_SPI->BANK[SpiBank].DATA;	// Void read	
		while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) {};		

		readVal = VOR_SPI->BANK[SpiBank].DATA;
		if(!(readVal & SPI_CMD_RDSR_WIP_Msk)){
			break;
    }

		VOR_SPI->BANK[SpiBank].FIFO_CLR = SPIC_FIFO_CLR_RXFIFO_Msk | SPIC_FIFO_CLR_TXFIFO_Msk; // Clear TX & RX FIFO

    if(SysTimeMs == (sysMsStart+2){
      break; // ensure loop terminates
    }

		for( delay=0; delay<5000; delay++ ){
			__NOP();
    }
	}
	VOR_SPI->BANK[SpiBank].FIFO_CLR = SPIC_FIFO_CLR_RXFIFO_Msk | SPIC_FIFO_CLR_TXFIFO_Msk; // Clear TX & RX FIFO
  #endif
	return;
}  

int CYP15B102_Init(unsigned long adr, unsigned long clk, unsigned long fnc, unsigned long bank) 
{
  volatile uint32_t voidRead;
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |=
        (CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_GPIO |
         CLK_ENABLE_IOCONFIG | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM); // enable at least these, plus which ever ones were already enabled
	
/*
	#ifdef IS_PPK
	defeat_WDT();
	#endif
*/
  
  if(bank > 2){
    SpiBank = INVALID_SPI_BANK;
    return 1; // invalid bank num
  }else{
    SpiBank = bank;
  }
  
  switch(SpiBank)
  {
    case 0:
#ifdef ROM1_SPIA_CS_NUM
      VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_SPIA;
      VOR_SPI->BANK[SpiBank].CTRL1 = (ROM1_SPIA_CS_NUM << SPI_PERIPHERAL_CTRL1_SS_Pos) |
                                                          SPI_PERIPHERAL_CTRL1_BLOCKMODE_Msk; // disable SPI Master mode, block xfer
      break;
#else
      return 1; // no external ROM1 on SPIA
#endif
    
    case 2:
      VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_SPIC;
      VOR_SPI->BANK[SpiBank].CTRL1 = (0UL << SPI_PERIPHERAL_CTRL1_SS_Pos) |
                                             SPI_PERIPHERAL_CTRL1_BLOCKMODE_Msk; // disable SPI Master mode, block xfer 
      break;
    
    default: 
      // nothing connected to SPI B
      return 1;
  }
   
  VOR_SPI->BANK[SpiBank].FIFO_CLR = SPIC_FIFO_CLR_RXFIFO_Msk | SPIC_FIFO_CLR_TXFIFO_Msk; // Clear TX & RX FIFO  
	
	VOR_SPI->BANK[SpiBank].CLKPRESCALE = 2; // Set fast clock rate   
	VOR_SPI->BANK[SpiBank].CTRL0 = (uint32_t)(3+1)<<SPI_PERIPHERAL_CTRL0_SCRDV_Pos | (8-1);   // Set clock pol/ph=00, clock div=03 and word size=7(8bit) 
	VOR_SPI->BANK[SpiBank].CTRL1 |= SPI_PERIPHERAL_CTRL1_ENABLE_Msk;  // Enable SPI
  VOR_SPI->BANK[SpiBank].FIFO_CLR = SPIC_FIFO_CLR_RXFIFO_Msk | SPIC_FIFO_CLR_TXFIFO_Msk; // Clear TX & RX FIFO  
	
	wait_idle();
	
#ifdef ROM1_WP_N_PIN
  // Write protect pin for external FRAM 
  if(SpiBank == 0)
  {
    // Setup WP_N pin (don't enable writes yet, though)
	  VOR_IOCONFIG->ROM1_WP_N_PORT[ROM1_WP_N_PIN] = (VOR_IOCONFIG->ROM1_WP_N_PORT[ROM1_WP_N_PIN] & ~((0x3)<<IOCONFIG_PORTB_FUNSEL_Pos))	| ((0)<<IOCONFIG_PORTB_FUNSEL_Pos);
	  ROM1_WP_N_BANK.DIR  |=1UL<<ROM1_WP_N_PIN;
    //ROM1_WP_N_BANK.CLROUT=1UL<<ROM1_WP_N_PIN;
    ROM1_WP_N_BANK.SETOUT=1UL<<ROM1_WP_N_PIN;
  }
#endif
	
  VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WREN | SPIC_DATA_BMSTART_BMSTOP_Msk; // Set Write Enable Latch(WEL) bit 


 	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_RDSR;	// Write single-byte Status Register message
	VOR_SPI->BANK[SpiBank].DATA = 0x00 | SPIC_DATA_BMSTART_BMSTOP_Msk;	// Clear the BP1/BP0 protection   


	#ifdef IS_TTZ
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_WRSR;	// Write two-byte Status Register message
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0x00;	// Clear the BP1/BP0 protection
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0x00 | SPIC_DATA_BMSTART_BMSTOP_Msk;	// Upper status reg word
  #else
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_WRSR;	// Write single-byte Status Register message
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0x00 | SPIC_DATA_BMSTART_BMSTOP_Msk;	// Clear the BP1/BP0 protection
	#endif

 	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_RDSR;	// Write single-byte Status Register message
	VOR_SPI->BANK[SpiBank].DATA = 0x00 | SPIC_DATA_BMSTART_BMSTOP_Msk;	// Clear the BP1/BP0 protection   

  wait_idle();
	
	return (0);
}

int CYP15B102_UnInit (unsigned long fnc) 
{
	wait_WIP();
	wait_idle();
  
  if(SpiBank == INVALID_SPI_BANK){
    return 1;
  }
	
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WREN | SPIC_DATA_BMSTART_BMSTOP_Msk; // Set Write Enable Latch(WEL) bit 
	wait_idle();	
	
	#ifdef IS_TTZ
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_WRSR;	// Write two-byte Status Register message
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0xfd;	// Clear the BP1/BP0 protection
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0x00 | SPIC_DATA_BMSTART_BMSTOP_Msk;	// Upper status reg word
  #else
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WRSR | SPIC_DATA_BMSTART_BMSTOP_Msk; // Set Write Enable Latch(WEL) bit 
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_WRSR;	// Write single-byte Status Register message
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)0xfd | SPIC_DATA_BMSTART_BMSTOP_Msk;	// set the BP1/BP0 protection and WP
	#endif
	wait_idle();
	
#ifdef ROM1_WP_N_PIN
	// Set WP to 0 to disable writes
  if(SpiBank == 0)
  {
    ROM1_WP_N_BANK.CLROUT=1UL<<ROM1_WP_N_PIN;
  }
#endif
  
  VOR_SPI->BANK[SpiBank].CTRL1 = 0; // disable SPI
  SpiBank = INVALID_SPI_BANK;
	
/*	
  VOR_IOCONFIG->PORTA[WD_FEED_PIN] = (VOR_IOCONFIG->PORTA[WD_FEED_PIN] & ~((0x3)<<IOCONFIG_PORTA_FUNSEL_Pos))	| ((FUNSEL0)<<IOCONFIG_PORTA_FUNSEL_Pos);
  WD_FEED_DEFEAT_TIM->ENABLE = 0x0;
  VOR_SYSCONFIG->TIM_CLK_ENABLE = (0UL);
*/	
  return (0);
}                               

int CYP15B102_EraseChip(void)
{
  uint32_t i, j, adr;
	uint32_t volatile voidRead;
  
  if(SpiBank == INVALID_SPI_BANK){
    return 1;
  }
  
	#ifdef USE_CHIP_ERASE
	wait_WIP();	
  wait_idle();
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WREN | SPIC_DATA_BMSTART_BMSTOP_Msk; // Send single "Write enable" byte  	
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)TTZ2564_CE | SPIC_DATA_BMSTART_BMSTOP_Msk;
  wait_idle();
	wait_WIP();	
	#else 
	//Byte-by-byte erasing, CANNOT cross page boundaries if device requires page writes
	//will require multiple page write ops
	//This will NOT work for Flash, which require ChipErase to clear memory to 0xff
	//Erase is not necessary for non-Flash types
	
	for( i=0; i<(CYP15B102_SZDEV/CYP15B102_SZPAGE); i++ ) {
		VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WREN | SPIC_DATA_BMSTART_BMSTOP_Msk; // Send single "Write enable" byte  	
	  wait_idle();
		
		adr=CYP15B102_SZPAGE*i;
		
		VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_WRITE;	// Write command 
	  VOR_SPI->BANK[SpiBank].DATA = MSB_ADDR_BYTE(adr); // Address high byte
	  VOR_SPI->BANK[SpiBank].DATA = MID_ADDR_BYTE(adr); // Address mid byte 
		VOR_SPI->BANK[SpiBank].DATA = LSB_ADDR_BYTE(adr); // Address low byte
				
		for( j=0; j<(CYP15B102_SZPAGE); j++) {
			while (!(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_TNF_Msk)) { };
			VOR_SPI->BANK[SpiBank].DATA = (uint32_t)(CYP15B102_VALEMPTY);
	  }
    while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_TNF_Msk) );
		VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPIC_DATA_BMSKIPDATA_Msk  | SPIC_DATA_BMSTART_BMSTOP_Msk; // Last Write also has End Block Transfer
				
	  wait_idle();
 		wait_WIP();
  }	
	#endif
  return(0);
	
}

int CYP15B102_EraseSector (unsigned long adr) 
{
  if(SpiBank == INVALID_SPI_BANK){
    return 1;
  }
	#ifdef CYP15B102_USE_SECTOR_ERASE
 	wait_WIP();		
	wait_idle();
  //Send new Write Enable
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WREN | SPIC_DATA_BMSTART_BMSTOP_Msk; // Set Write Enable Latch(WEL) bit before sector erase   	
  wait_idle();
	
	// Send Sector Erase	
  VOR_SPI->BANK[SpiBank].DATA = (uint32_t)TTZ2564_SE;	// Send sector erase command  
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)MSB_ADDR_BYTE(adr); // Address high byte
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)MID_ADDR_BYTE(adr); // Address mid byte 
  VOR_SPI->BANK[SpiBank].DATA = (uint32_t)LSB_ADDR_BYTE(adr) | SPIC_DATA_BMSTART_BMSTOP_Msk; // Address low byte
	
 	wait_WIP();		
	wait_idle();
	#endif
  return (0);                                  
}

int CYP15B102_ProgramPage(unsigned long adr, unsigned long sz, unsigned char *buf) { 
	int32_t volatile voidRead;
  
  if(SpiBank == INVALID_SPI_BANK){
    return 1;
  }

#ifdef ROM1_WP_N_PIN
  if(SpiBank == 0){
	  //set WP_N to 1 to enable writes	
	  ROM1_WP_N_BANK.DIR  |=1UL<<ROM1_WP_N_PIN;
	  ROM1_WP_N_BANK.SETOUT=1UL<<ROM1_WP_N_PIN;
  }
#endif
	
 	wait_WIP();
  wait_idle();
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPI_CMD_WREN | SPIC_DATA_BMSTART_BMSTOP_Msk; // Set Write Enable Latch(WEL) bit 
	
	wait_idle();
	
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_WRITE; // Write command
	VOR_SPI->BANK[SpiBank].DATA = MSB_ADDR_BYTE(adr); // Address high byte
	VOR_SPI->BANK[SpiBank].DATA = MID_ADDR_BYTE(adr); // Address mid byte 
	VOR_SPI->BANK[SpiBank].DATA = LSB_ADDR_BYTE(adr); // Address low byte
		
  while( sz-- ) {
		while (!(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_TNF_Msk)) {};
	  VOR_SPI->BANK[SpiBank].DATA = (unsigned char)(*buf++);
	  voidRead=VOR_SPI->BANK[SpiBank].DATA;
  }
	while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_TNF_Msk) );
	VOR_SPI->BANK[SpiBank].DATA = (uint32_t)SPIC_DATA_BMSKIPDATA_Msk  | SPIC_DATA_BMSTART_BMSTOP_Msk;
		
  wait_idle();
 	wait_WIP();		
  
  if(SpiBank == 0){
    // Clear WP to disable writes
    //ROM1_WP_N_BANK.CLROUT=1UL<<ROM1_WP_N_PIN;
  }
	
  return(0);
}

int CYP15B102_BlankCheck(unsigned long adr, unsigned long sz, unsigned char pat)
{
  // not implemented
  return (0);       
}

unsigned long CYP15B102_Verify(unsigned long adr, unsigned long sz, unsigned char *buf)
{
	int32_t i;
	unsigned char readVal;
  
  if(SpiBank == INVALID_SPI_BANK){
    return 0;
  }
  
 	wait_WIP();
	wait_idle();
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_READ; // Read command 
	VOR_SPI->BANK[SpiBank].DATA = MSB_ADDR_BYTE(adr); // Address high byte
	VOR_SPI->BANK[SpiBank].DATA = MID_ADDR_BYTE(adr); // Address mid byte 
	VOR_SPI->BANK[SpiBank].DATA = LSB_ADDR_BYTE(adr); // Address low byte

  for( i=0; i<4; i++ ) {
		VOR_SPI->BANK[SpiBank].DATA = 0x00; // Pump the SPI
	  while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) { };
	  readVal =	VOR_SPI->BANK[SpiBank].DATA; // Void read
  }
	
	for( i=0; i<sz; i++) {
	  VOR_SPI->BANK[SpiBank].DATA = 0x00; // Pump the SPI
		while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) { };
	  readVal = VOR_SPI->BANK[SpiBank].DATA; 
	  if(*buf != readVal) {
			while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) { };
			return((i<<24)|(*buf<<16)|(readVal<<8)| (VOR_SPI->BANK[SpiBank].DATA));
		}
		buf++;
	}
	VOR_SPI->BANK[SpiBank].DATA = SPIC_DATA_BMSTART_BMSTOP_Msk; // Terminate Block Transfer
	wait_idle();
		
	return(adr+sz);
}

unsigned long CYP15B102_Read(unsigned long adr, unsigned long sz, unsigned char *buf)
{
	int32_t i;
	unsigned char readVal;
  
  if(SpiBank == INVALID_SPI_BANK){
    return 0;
  }
  
 	wait_WIP();
	wait_idle();
	VOR_SPI->BANK[SpiBank].DATA = SPI_CMD_READ; // Read command 
	VOR_SPI->BANK[SpiBank].DATA = MSB_ADDR_BYTE(adr); // Address high byte
	VOR_SPI->BANK[SpiBank].DATA = MID_ADDR_BYTE(adr); // Address mid byte 
	VOR_SPI->BANK[SpiBank].DATA = LSB_ADDR_BYTE(adr); // Address low byte

  for( i=0; i<4; i++ ) {
		VOR_SPI->BANK[SpiBank].DATA = 0x00; // Pump the SPI
	  while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) { };
	  readVal =	VOR_SPI->BANK[SpiBank].DATA; // Void read
  }
	
	for( i=0; i<sz; i++) {
	  VOR_SPI->BANK[SpiBank].DATA = 0x00; // Pump the SPI
		while( !(VOR_SPI->BANK[SpiBank].STATUS & SPIC_STATUS_RNE_Msk) ) { };
	  readVal = VOR_SPI->BANK[SpiBank].DATA; 
    *buf = readVal;
		buf++;
	}
	VOR_SPI->BANK[SpiBank].DATA = SPIC_DATA_BMSTART_BMSTOP_Msk; // Terminate Block Transfer
	wait_idle();
		
	return(adr+sz);
}
